/*
 * dspin.cpp
 *
 * (for documentation, refer to dspin.h file)
 *
 *  Created on: Jun 16, 2012
 *      Author: Eric Pascual (eric@pobot.org)
 */

#include "dspin.h"

#define read_param(p) handle_param((p), 0)
#define write_param(p,v) handle_param((p), (v))

#define spi_read() spi_send(0)

#define MASK_4	0x00000F
#define MASK_5	0x00001F
#define MASK_7	0x00007F
#define MASK_8	0x0000FF
#define MASK_9	0x0001FF
#define MASK_10	0x0003FF
#define MASK_12	0x000FFF
#define MASK_13	0x001FFF
#define MASK_14	0x003FFF
#define MASK_16	0x00FFFF
#define MASK_20	0x0FFFFF
#define MASK_22	0x3FFFFF

typedef struct {
	uint8_t len_bytes;
	uint32_t mask;
} REG_DESCR, *REG_DESCR_PTR;

static REG_DESCR reg_descr[] = { { 0, 0 }, // reg 0 does not exist
		{ 3, MASK_22 }, // dSPIN_REG_ABS_POS
		{ 2, MASK_9 }, // dSPIN_REG_EL_POS
		{ 3, MASK_22 }, // dSPIN_REG_MARK
		{ 3, MASK_20 }, // dSPIN_REG_SPEED
		{ 2, MASK_12 }, // dSPIN_REG_ACC
		{ 2, MASK_12 }, // dSPIN_REG_DEC
		{ 2, MASK_10 }, // dSPIN_REG_MAX_SPEED
		{ 2, MASK_13 }, // dSPIN_REG_MIN_SPEED
		{ 1, MASK_8 }, // dSPIN_REG_KVAL_HOLD
		{ 1, MASK_8 }, // dSPIN_REG_KVAL_RUN
		{ 1, MASK_8 }, // dSPIN_REG_KVAL_ACC
		{ 1, MASK_8 }, // dSPIN_REG_KVAL_DEC
		{ 2, MASK_14 }, // dSPIN_REG_INT_SPD
		{ 1, MASK_8 }, // dSPIN_REG_ST_SLP
		{ 1, MASK_8 }, // dSPIN_REG_FN_SLP_ACC
		{ 1, MASK_8 }, // dSPIN_REG_FN_SLP_DEC
		{ 1, MASK_4 }, // dSPIN_REG_K_THERM
		{ 1, MASK_5 }, // dSPIN_REG_ADC_OUT
		{ 1, MASK_4 }, // dSPIN_REG_OCD_TH
		{ 1, MASK_7 }, // dSPIN_REG_STALL_TH
		{ 2, MASK_10 }, // dSPIN_REG_FS_SPD
		{ 1, MASK_8 }, // dSPIN_REG_STEP_MODE
		{ 1, MASK_8 }, // dSPIN_REG_ALARM_EN
		{ 2, MASK_16 }, // dSPIN_REG_CONFIG
		{ 2, MASK_16 } // dSPIN_REG_STATUS
};

DSpin::DSpin(PinName mosi, PinName miso, PinName sclk, PinName cs, PinName stby) :
	_cs(DigitalOut(cs)), _stby(DigitalOut(stby)), _spi(SPI(mosi, miso, sclk)) {
	_not_busy = NULL;
	_cs = 1; // unselect the chip
}

DSpin::DSpin(PinName mosi, PinName miso, PinName sclk, PinName cs, PinName stby, PinName not_busy) :
	_cs(DigitalOut(cs)), _stby(DigitalOut(stby)), _spi(SPI(mosi, miso, sclk)) {
	_not_busy = new DigitalIn(not_busy);
	_cs = 1; // unselect the chip
}

DSpin::~DSpin() {
}

void DSpin::init() {
	_spi.format(8, 3);

	// reset the chip
	_stby = 1;
	wait_ms(1);
	_stby = 0;
	wait_ms(1);
	_stby = 1;
	wait_ms(1);
}

uint32_t DSpin::get_status() {
	int res = 0;
	spi_send( dSPIN_CMD_GET_STATUS);
	res = spi_read() << 8;
	res |= spi_read();
	return res;
}

uint32_t DSpin::get_config() {
	return get_register(dSPIN_REG_CONFIG);
}

uint32_t DSpin::get_register(dSPIN_REGISTER reg) {
	REG_DESCR_PTR reg_decr_p = &reg_descr[reg];

	spi_send(dSPIN_CMD_GET_PARAM | reg);

	uint32_t value = 0;
	switch (reg_decr_p->len_bytes) {
	case 3:
		value |= spi_read() << 16;
	case 2:
		value |= spi_read() << 8;
	case 1:
		value |= spi_read();
	}
	return value;
}

void DSpin::set_register(dSPIN_REGISTER reg, uint32_t value) {
	REG_DESCR_PTR reg_decr_p = &reg_descr[reg];

	if (value > reg_decr_p->mask)
		value = reg_decr_p->mask;

	spi_send(dSPIN_CMD_SET_PARAM | reg);

	switch (reg_decr_p->len_bytes) {
	case 3:
		spi_send((uint8_t) (value >> 16));
	case 2:
		spi_send((uint8_t) (value >> 8));
	case 1:
		spi_send((uint8_t) value);
	}
}

inline uint32_t DSpin::get_current_speed() {
	uint32_t steps_per_tick = get_register(dSPIN_REG_SPEED);
	return (uint32_t)(steps_per_tick * 0.0149);
}

void DSpin::enable_low_speed_optimization(bool enabled) {
	set_register(dSPIN_REG_MIN_SPEED, enabled ? 0x1000 : 0);
}

void DSpin::step_clock(uint8_t dir) {
	spi_send(dSPIN_CMD_STEP_CLOCK | dir);
}

void DSpin::move(uint32_t n_step, dSPIN_DIRECTION dir) {
	spi_send(dSPIN_CMD_MOVE | (dir & 0x01));
	spi_send3(n_step, MASK_22);
}

void DSpin::run(uint32_t stepsPerSec, dSPIN_DIRECTION dir) {
	spi_send(dSPIN_CMD_RUN | (dir & 0x01));
	spi_send3(speed_calc(stepsPerSec), MASK_20);
}

void DSpin::goto_pos(uint32_t pos) {
	spi_send( dSPIN_CMD_GOTO);
	spi_send3(pos, MASK_22);
}

void DSpin::goto_pos(uint32_t pos, dSPIN_DIRECTION dir) {
	spi_send(dSPIN_CMD_GOTO_DIR | (dir & 0x01));
	spi_send3(pos, MASK_22);
}

void DSpin::go_until_switch(dSPIN_ACTION action, dSPIN_DIRECTION dir, uint32_t stepsPerSec) {
	spi_send(dSPIN_CMD_GO_UNTIL | action | dir);
	spi_send3(speed_calc(stepsPerSec), MASK_22);
}

void DSpin::release_switch(dSPIN_ACTION action, dSPIN_DIRECTION dir) {
	spi_send(dSPIN_CMD_RELEASE_SW | action | dir);
}

void DSpin::go_home() {
	spi_send(dSPIN_CMD_GO_HOME);
}

void DSpin::reset_pos() {
	spi_send(dSPIN_CMD_RESET_POS);
}

void DSpin::go_mark() {
	spi_send(dSPIN_CMD_GO_MARK);
}

void DSpin::reset_device() {
	spi_send(dSPIN_CMD_RESET_DEVICE);
}

void DSpin::soft_stop() {
	spi_send(dSPIN_CMD_SOFT_STOP);
}

void DSpin::hard_stop() {
	spi_send(dSPIN_CMD_HARD_STOP);
}

void DSpin::soft_hiZ() {
	spi_send(dSPIN_CMD_SOFT_HIZ);
}

void DSpin::hard_hiZ() {
	spi_send(dSPIN_CMD_HARD_HIZ);
}

void DSpin::wait_untill_not_busy() {
	if (_not_busy != NULL) {
		while (_not_busy->read() == 0) {
			wait_ms(1);
		}
	} else {
		printf("[WARN] busy pin not set\r\n");
	}
}

bool DSpin::is_busy() {
	return _not_busy != NULL ? !_not_busy->read() : false;
}

long DSpin::get_absolute_position() {
	return get_register(dSPIN_REG_ABS_POS);
}

void DSpin::set_max_speed(float stepsPerSec) {
	set_register(dSPIN_REG_MAX_SPEED, max_speed_calc(stepsPerSec));
}

void DSpin::set_acceleration(float stepsPerSec2) {
	set_register(dSPIN_REG_ACC, acc_dec_calc(stepsPerSec2));
}

void DSpin::set_deceleration(float stepsPerSec2) {
	set_register(dSPIN_REG_DEC, acc_dec_calc(stepsPerSec2));
}

uint8_t DSpin::spi_send(uint8_t data) {
	_cs = 0;
	uint8_t data_out = _spi.write(data);
	_cs = 1;
	wait_us(1);
	return data_out;
}

void DSpin::spi_send3(uint32_t v, uint32_t max_value) {
	if (v > max_value)
		v = max_value;
	spi_send((uint8_t) (v >> 16));
	spi_send((uint8_t) (v >> 8));
	spi_send((uint8_t) (v));
}

uint32_t DSpin::max_speed_calc(float stepsPerSec) {
	float temp = stepsPerSec * .065536; // .065536 = 250ns / 2^-18
	if ((uint32_t) long(temp) > MASK_10)
		return MASK_10;
	else
		return (uint32_t) long(temp);
}

uint32_t DSpin::fs_threshold_calc(float stepsPerSec) {
	float temp = (stepsPerSec * .065536) - .5;
	if ((uint32_t) long(temp) > MASK_10)
		return MASK_10;
	else
		return (uint32_t) long(temp);
}

uint32_t DSpin::speed_calc(float stepsPerSec) {
	float temp = stepsPerSec * 67.106;
	if ((uint32_t) long(temp) > MASK_20)
		return MASK_20;
	else
		return (uint32_t) temp;
}

uint16_t DSpin::acc_dec_calc(float stepsPerSec2) {
	float temp = stepsPerSec2 * .06872;	// .06872 = 250ns ^ 2 / 2^-40
	if ((long(temp) > MASK_12))
		return MASK_12;
	else
		return (uint16_t) temp;
}

