/** \file dspin.h
 *
 * Defines the DSpin class, which provides a convenient interface to STMicro L6470
 * stepper driver.
 *
 *  Created on: Jun 16, 2012
 *      Author: Eric Pascual (eric@pobot.org)
 */

#ifndef DSPIN_H_
#define DSPIN_H_

#include "mbed.h"
#include "dspin_defs.h"

/**
 * DSpin interface class.
 */
class DSpin {
public:
	/**
	 * Full featured constructor.
	 *
	 * \param[in]	mosi		the pin name used for the MOSI (SDI) signal
	 * \param[in]	miso		the pin name used for the MISO (SDO) signal
	 * \param[in]	sclk		the pin name used for the SCLK (CK) signal
	 * \param[in]	cs			the	pin name used for the chip select (CSN) signal
	 * \param[in]	stby		the	pin name used for the standby (STDBY) signal
	 * \param[in]	not_busy	the pin name used for the not busy (BUSYN) signal
	 */
	DSpin(PinName mosi, PinName miso, PinName sclk, PinName cs, PinName stby, PinName not_busy);

	/**
	 * Alternate constructor, without monitoring of the busy state.
	 *
	 * \param[in]	mosi		the pin name used for the MOSI (SDI) signal
	 * \param[in]	miso		the pin name used for the MISO (SDO) signal
	 * \param[in]	sclk		the pin name used for the SCLK (CK) signal
	 * \param[in]	cs			the	pin name used for the chip select (CSN) signal
	 * \param[in]	stby		the	pin name used for the standby (STDBY) signal
	 */
	DSpin(PinName mosi, PinName miso, PinName sclk, PinName cs, PinName stby);

	virtual ~DSpin();

	/** @name Initialization, reset and information methods
	 */

	///@{
	/**
	 * Initialization of the chip.
	 *
	 * Must be called before any operation.
	 */
	void init();

	/**
	 * Retrieves the current content of the status register
	 */
	uint32_t get_status();

	/**
	 * Retrieves the current content of the configuration register
	 */
	uint32_t get_config();

	/**
	 * Resets the device.
	 *
	 * Cycles through standby states as described in the datasheet.
	 */
	void reset_device();

	/**
	 * Resets the position register to 0.
	 *
	 * In other words, it sets the home position to the current one.
	 */
	void reset_pos();

	/**
	 * Returns the absolute position (in steps).
	 */
	long get_absolute_position();

	/**
	 * Returns the current motor speed. (in steps per tick)
	 */
	uint32_t get_current_speed();

	///@}

	/** @name Configuration methods
	 */

	///@{
	/**
	 * Sets the maximum speed, in steps per second.
	 */
	void set_max_speed(float stepsPerSec);

	/**
	 * Sets the acceleration, in steps per second^2.
	 */
	void set_acceleration(float stepsPerSec2);

	/**
	 * Sets the deceleration, in steps per second^2.
	 */
	void set_deceleration(float stepsPerSec2);

	/**
	 * Sets the status of the low speed optimization feature.
	 *
	 * \param[in] enabled	true/false depending on desired enabling state
	 */
	void enable_low_speed_optimization(bool enabled);

	///@}

	/** @name Movements control
	 */

	///@{
	/**
	 * Moves the motor a given distance away from current one, using an explicit direction
	 *
	 * \param[in] n_step	number of steps
	 * \param[in] dir		direction of move
	 */
	void move(uint32_t n_step, dSPIN_DIRECTION dir);

	/**
	 * Continuously runs the motor with a given speed and direction.
	 *
	 * \param[in] stepsPerSec	the speed, in steps per second
	 * \param[in] dir			direction of rotation
	 */
	void run(uint32_t stepsPerSec, dSPIN_DIRECTION dir);

	/**
	 * Stops the motor with deceleration.
	 *
	 * Motor is left in braking mode (ie coils energized).
	 */
	void soft_stop();

	/**
	 * Immediately stops the motor without any deceleration.
	 *
	 * Motor is left in braking mode (ie coils energized).
	 */
	void hard_stop();

	/**
	 * Stops the motor with deceleration, and disconnect the coils.
	 *
	 * Motor is left in floating mode.
	 */
	void soft_hiZ();

	/**
	 * Immediately stops the motor without any deceleration, and disconnect the coils.
	 *
	 * Motor is left in floating mode.
	 */
	void hard_hiZ();

	/**
	 * Moves the motor to an absolute position, using appropriate direction.
	 *
	 * \param[in] pos		target position (in steps)
	 */
	void goto_pos(uint32_t pos);

	/**
	 * Moves the motor to an absolute position, using an explicit direction.
	 *
	 * Note that this can result in no move if current position, target position and
	 * direction are not compatible.
	 *
	 * \param[in] pos		target position (in steps)
	 * \param[in] dir		direction of move
	 */
	void goto_pos(uint32_t pos, dSPIN_DIRECTION dir);

	/**
	 * Runs the motor until the switch closes.
	 *
	 * \param[in] action		reset position or copy it in mark register when switch closes
	 * \param[in] dir			direction of move
	 * \param[in] stepsPerSec	rotation speed (in steps per second)
	 */
	void go_until_switch(dSPIN_ACTION action, dSPIN_DIRECTION dir, uint32_t stepsPerSec);

	/**
	 * Runs the motor until the switch is released.
	 *
	 * Will use the current setting of motor minimum speed.
	 *
	 * \param[in] action	see go_until_switch()
	 * \param[in] dir		direction of move
	 */
	void release_switch(dSPIN_ACTION action, dSPIN_DIRECTION dir) ;

	/**
	 * Returns the motor to the home position.
	 *
	 * Will use the current setting of motor maximum speed and appropriate direction.
	 */
	void go_home();

	/**
	 * Moves the motor to the position stored in the mark register.
	 *
	 * Will use the current setting of motor maximum speed and appropriate direction.
	 */
	void go_mark();

	/**
	 * Advances the motor 1 step in the given direction
	 *
	 * \param[in] dir		direction of move
	 */
	void step_clock(unsigned char dir);
	///@}

	/** @name Busy state management
	 */

	///@{
	/**
	 * Waits until the busy signal is reset.
	 *
	 *	Returns immediately if the busy signal is not monitored.
	 */
	void wait_untill_not_busy();

	/**
	 * Returns the current state of the busy signal.
	 *
	 *	Always returns false if the busy signal is not monitored.
	 */
	bool is_busy();
	///@}
	///@}

	/** @name Low-level register access
	 */

	///@{
	/**
	 * Returns the content of a register.
	 *
	 * \param[in] reg	the register
	 */
	uint32_t get_register(dSPIN_REGISTER reg);

	/**
	 * Sets the content of a register.
	 *
	 * The register size is taken in account to clamp the provided value if needed.
	 *
	 * \param[in] reg	the register
	 * \param[in] value	the new value
	 */
	void set_register(dSPIN_REGISTER reg, uint32_t value);
	///@}

	/** @name Helper methods
	 */
	///@{
	/**
	 * Computes the register value corresponding to a given speed.
	 */
	static uint32_t speed_calc(float stepsPerSec);

	/**
	 * Computes the register value corresponding to a given maximum speed.
	 */
	static uint32_t max_speed_calc(float stepsPerSec);

	/**
	 * Computes the register value corresponding to a given threshold.
	 */
	static uint32_t fs_threshold_calc(float stepsPerSec);

	/**
	 * Computes the register value corresponding to an acceleration or
	 * deceleration rate given in steps/s2.
	 */
	static uint16_t acc_dec_calc(float stepsPerSec2);
	///@}

private:
	DigitalOut _cs;
	DigitalOut _stby;
	DigitalIn* _not_busy;
	SPI _spi;

	uint8_t spi_send(uint8_t data);
	void spi_send3(uint32_t v, uint32_t max);
};

#endif /* DSPIN_H_ */


