#include "mbed.h"
#include "dspin.h"
#include <ctype.h>

DSpin dspin(p5, p6, p7, p8, p11, p12);

DigitalOut led1(LED1);
DigitalOut led2(LED2);
DigitalOut led3(LED3);
DigitalOut led4(LED4);

Ticker busy_led_updater;

void fatal_flash(DigitalOut led) {
	while (1) {
		led = !led;
		wait(0.5);
	}
}

void update_busy_led() {
	led4 = dspin.is_busy();
}

void display_help() {
	printf("Available commands:\n\r"
			"1..9\tgoto absolute position (n * 100 steps)\r\n"
			"+\tmove 1 turn forward\r\n"
			"-\tmove 1 turn backward\r\n"
			"0\tgo back home position\r\n"
			"f\trun forward continuously\r\n"
			"r\trun backward (reverse) continuously\r\n"
			"w\twipe move\r\n"
			"s\tsoft stop\r\n"
			"b\tbrake (hard-stop)\r\n"
			"z\tput driver in hi-Z state\r\n"
			"\n\r");
}

int main() {
	printf("\r\n"
		"\r\n"
		"---------------------------------\r\n"
		"       dSPIN and mBed demo\r\n"
		"       (for a 400s/r stepper)\r\n"
		"---------------------------------\r\n"
		"\r\n");

	printf("[INFO] dSPIN initialization... ");
	fflush(stdout);
	dspin.init();

	unsigned short config = dspin.get_config();
	if (config != dSPIN_FACT_RESET_CONFIG) {
		printf("\r\n[ERR] dSPIN unexpected configuration : 0x%x\r\n", config);
		fatal_flash(led1);
	}
	printf("ok\r\n");
	led1 = 1;

	unsigned short status = dspin.get_status();
	int acc = dspin.get_register(dSPIN_REG_ACC);
	int dec = dspin.get_register(dSPIN_REG_DEC);

	printf("[INFO] initial configuration : CF=0x%.4x ST=0x%.4x ACC=0x%.2x  DEC=0x%.2x\r\n", config, status, acc, dec);

	//	dspin.set_param(dSPIN_STEP_MODE, !dSPIN_SYNC_EN | dSPIN_STEP_SEL_1_8 | dSPIN_SYNC_SEL_1);
	//	dspin.set_param(dSPIN_MAX_SPEED, DSpin::max_speed_calc(1000));
	//	dspin.set_param(dSPIN_FS_SPD, DSpin::fs_threshold_calc(500));
	//	dspin.set_param(dSPIN_ACC, 0xff);
	//	dspin.set_param(dSPIN_OCD_TH, dSPIN_OCD_TH_1125mA);
	//	dspin.set_param(dSPIN_CONFIG, dSPIN_CONFIG_PWM_DIV_1 | dSPIN_CONFIG_PWM_MUL_2 | dSPIN_CONFIG_SR_290V_us
	//			| dSPIN_CONFIG_OC_SD_ENABLE | dSPIN_CONFIG_VS_COMP_DISABLE | dSPIN_CONFIG_SW_HARD_STOP
	//			| dSPIN_CONFIG_INT_16MHZ);
	//	dspin.set_param(dSPIN_KVAL_RUN, 0xFF);
	//	dspin.get_status();

	config = dspin.get_config();
	printf("[INFO] current configuration : CF=0x%.4x\r\n\r\n\r\n", config);

	led2 = 1;

	busy_led_updater.attach_us(&update_busy_led, 1000);
	led3 = 1;

	while (1) {
		printf("[CMD] (? for help) > ");
		fflush(stdout);

		char c = toupper(fgetc(stdin));
		printf("%c\r\n", c);

		switch (c) {
		case '?':
			display_help();
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9': {
			long target_pos = 100 * 128 * (c - '0');
			if (dspin.get_absolute_position() != target_pos) {
				dspin.set_max_speed(1000);
				printf("[INFO] goto_pos(%d)... ", (c - '0') * 100);
				fflush(stdout);

				dspin.goto_pos(target_pos);
				dspin.wait_untill_not_busy();
				printf("done\r\n");
			} else {
				printf("[WARN] already at target position\r\n");
			}
		}
			break;

		case '+':
		case '-': {
			long steps = 400 * 128;
			dspin.set_max_speed(1000);
			printf("[INFO] move(400, %s)... ", c == '+' ? "FWD" : "REV");
			fflush(stdout);
			dspin.move(steps, c == '+' ? dSPIN_DIR_FWD : dSPIN_DIR_REV);
			dspin.wait_untill_not_busy();
			printf("done\r\n");
		}
			break;

		case '0':
			if (dspin.get_absolute_position() != 0) {
				dspin.set_max_speed(1200);
				printf("[INFO] go_home()... ");
				fflush(stdout);
				dspin.go_home();
				dspin.wait_untill_not_busy();
				printf("done\r\n");
			} else {
				printf("[WARN] already at home position\r\n");
			}
			break;

		case 'W': {
			dspin.set_max_speed(100);
			dSPIN_DIRECTION dir = dSPIN_DIR_FWD;
			for (int i = 0; i < 4; i++) {
				printf("[INFO] move(100, %s)... ", dir ? "FDW" : "REV");
				fflush(stdout);
				dspin.move(100 * 128, dir);
				dspin.wait_untill_not_busy();
				printf("done\r\n");
				dir = (dir == dSPIN_DIR_FWD) ? dSPIN_DIR_REV : dSPIN_DIR_FWD;
			}
		}
			break;

		case 'F':
			printf("[INFO] run(FWD)...\r\n");
			dspin.set_max_speed(1200);
			dspin.run(1000, dSPIN_DIR_FWD);
			break;

		case 'R':
			printf("[INFO] run(REV)...\r\n");
			dspin.set_register(dSPIN_REG_MAX_SPEED, DSpin::max_speed_calc(1200));
			dspin.run(500, dSPIN_DIR_REV);
			break;

		case 'S':
			printf("[INFO] soft_stop()...\r\n");
			dspin.soft_stop();
			break;

		case 'B':
			printf("[INFO] hard_stop()...\r\n");
			dspin.hard_stop();
			break;

		case 'Z':
			printf("[INFO] hard_hiZ()...\r\n");
			dspin.hard_hiZ();
			break;

		default:
			printf("[ERROR] invalid command\r\n");
		}

		printf("\n");
	}
}

